#auto-generated perl error module for .
package ERRORS;
require Exporter;
our @ISA = ("Exporter");
$E_LISTEXISTS="Mailing List '%(list)s' already exists. ";
$E_MISSINGLIST="Invalid or missing list name.";
$E_ALIASEXISTS="A mail alias for '%(alias)s' already exists.";
$E_MISSINGOWNER="Invalid or missing owner's email address.";
$E_MISSINGADMINPASSWORD="Invalid or missing administration password.";
$E_MISSINGMODERATOR="Invalid or missing moderator's email address.";
$E_LISTNAMEEXISTS="The mailing list could not be created. Please use a different \"List Name\". The list name should not use the user names of site users.";
our @EXPORT = qw( $E_LISTEXISTS $E_MISSINGLIST $E_ALIASEXISTS $E_MISSINGOWNER $E_MISSINGADMINPASSWORD $E_MISSINGMODERATOR $E_LISTNAMEEXISTS);
1;
